using System;
using System.Collections;

namespace Sudoku
{
	/// <summary>
	/// 
	/// </summary>
	public class Puzzle
	{
		protected const int puzzleSize = 9;  // Should always be a square: 4, 9, 16, 25, 36
		protected const int sqrtPuzzleSize = 3;

		protected PuzzleCell [] cells;
		protected PuzzleBlock [] blocks;
		protected PuzzleRow [] rows;
		protected PuzzleColumn [] columns;

		public Puzzle()
		{
			cells = new PuzzleCell [puzzleSize * puzzleSize];
			blocks = new PuzzleBlock [puzzleSize];
			rows = new PuzzleRow [puzzleSize];
			columns = new PuzzleColumn [puzzleSize];
		}

		protected void InitializeCells(int [] values) 
		{
			int index = 0;
			foreach (int value1 in values) 
			{
				cells[index++] = new PuzzleCell(value1);
			}
		}

		protected void InitializeBlocks() 
		{
			for (int cellIndex = 0; cellIndex < cells.Length; cellIndex++)  
			{
				int blockRow = cellIndex / (puzzleSize * sqrtPuzzleSize);
				int blockColumn = (cellIndex % puzzleSize) / sqrtPuzzleSize;
				int blockIndex = blockRow * sqrtPuzzleSize + blockColumn;
				if (null == blocks[blockIndex]) 
				{
					blocks[blockIndex] = new PuzzleBlock();
				}
				blocks[blockIndex].AddCell(cells[cellIndex]);
			}
		}

		protected void InitializeRows() 
		{
			for (int cellIndex = 0;  cellIndex < cells.Length;  cellIndex++)
			{
				int rowIndex = cellIndex / puzzleSize;
				if (null == rows[rowIndex])
				{
					rows[rowIndex] = new PuzzleRow();
				}
				rows[rowIndex].AddCell(cells[cellIndex]);
			}
		}

		protected void InitializeColumns()
		{
			for (int cellIndex = 0;  cellIndex < cells.Length;  cellIndex++)
			{
				int columnIndex = cellIndex % puzzleSize;
				if (null == columns[columnIndex])
				{
					columns[columnIndex] = new PuzzleColumn();
				}
				columns[columnIndex].AddCell(cells[cellIndex]);
			}
		}

		public void Initialize(int [] values) 
		{
			if (values.Length == cells.Length) 
			{
				InitializeCells(values);
				InitializeBlocks();
				InitializeRows();
				InitializeColumns();
			}
		}

		public void Advance() 
		{
			AdvancePush();
			AdvancePull();
		}

		public void AdvancePush()
		{
			foreach (PuzzleCell c in cells) 
			{
				if (c.IsCertain) 
				{
					c.Row.Remove(c);
					c.Column.Remove(c);
					c.Block.Remove(c);
				}
			}
		}

		public void AdvancePull()
		{
			for (int values = 1;  values < 10;  values++) 
			{
				foreach (PuzzleRow row in rows) 
				{
					row.Pull(values);
				}
				foreach (PuzzleColumn column in columns)
				{
					column.Pull(values);
				}
				foreach (PuzzleBlock block in blocks)
				{
					block.Pull(values);
				}
			}
		}

		public bool Solved
		{
			get
			{
				foreach (PuzzleCell c in cells)
				{
					if (!c.IsCertain)
						return false;
				}
				return true;
			}
		}
	}
}
